package com.ruoyi.system.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.system.domain.TPatientHospitalDoctor;
import com.ruoyi.system.domain.TRecord;
import com.ruoyi.system.domain.req.TRecordResultReq;
import com.ruoyi.system.domain.resp.RecordHistoryDataResp;
import com.ruoyi.system.domain.resp.TRecordResp;
import com.ruoyi.system.mapper.TRecordMapper;
import com.ruoyi.system.service.ITRecordService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 诊断记录信息Service业务层处理
 *
 * @author ruoyi
 * @date 2022-08-03
 */
@Service
public class TRecordServiceImpl implements ITRecordService
{
    @Autowired
    private TRecordMapper tRecordMapper;

    /**
     * 查询诊断记录信息
     *
     * @param id 诊断记录信息主键
     * @return 诊断记录信息
     */
    @Override
    public TRecord selectTRecordById(Long id)
    {
        return tRecordMapper.selectTRecordById(id);
    }

    /**
     * 查询诊断记录信息列表
     *
     * @param tRecord 诊断记录信息
     * @return 诊断记录信息
     */
    @Override
    public List<TRecord> selectTRecordList(TRecord tRecord)
    {
        return tRecordMapper.selectTRecordList(tRecord);
    }

    /**
     * 新增诊断记录信息
     *
     * @param tRecord 诊断记录信息
     * @return 结果
     */
    @Override
    public int insertTRecord(TRecord tRecord)
    {
        tRecord.setCreateTime(DateUtils.getNowDate());
        return tRecordMapper.insertTRecord(tRecord);
    }

    /**
     * 修改诊断记录信息
     *
     * @param tRecord 诊断记录信息
     * @return 结果
     */
    @Override
    public int updateTRecord(TRecord tRecord)
    {
        // 处理无图片信息
        if (tRecord.getAiResult() != null) {
            JSONObject jsonObject = JSONObject.parseObject(tRecord.getAiResult());
            tRecord.setAiResult2(noPicture(jsonObject));
        }
        if (tRecord.getUpdateResult() != null) {
            JSONObject jsonObject = JSONObject.parseObject(tRecord.getUpdateResult());
            tRecord.setUpdateResult2(noPicture(jsonObject));
        }
        tRecord.setUpdateTime(DateUtils.getNowDate());
        return tRecordMapper.updateTRecord(tRecord);
    }

    private static String noPicture(JSONObject jsonObject) {
        jsonObject.getJSONObject("shezhi").getJSONObject("shemian").put("img_sz", null);
        jsonObject.getJSONObject("shezhi").getJSONObject("yudian").put("img_sz_yudian", null);
        jsonObject.getJSONObject("shezhi").getJSONObject("yuban").put("img_sz_yuban", null);
        jsonObject.getJSONObject("shezhi").getJSONObject("liewen").put("img_sz_liewen", null);
        jsonObject.getJSONObject("shetai").put("img_st", null);
        jsonObject.getJSONObject("shexia").put("img_sx", null);
        jsonObject.getJSONObject("shexia").put("img_sx_vein", null);
        return jsonObject.toString();
    }

    /**
     * 批量删除诊断记录信息
     *
     * @param ids 需要删除的诊断记录信息主键
     * @return 结果
     */
    @Override
    public int deleteTRecordByIds(Long[] ids)
    {
        return tRecordMapper.deleteTRecordByIds(ids);
    }

    /**
     * 删除诊断记录信息信息
     *
     * @param id 诊断记录信息主键
     * @return 结果
     */
    @Override
    public int deleteTRecordById(Long id)
    {
        return tRecordMapper.deleteTRecordById(id);
    }

    @Override
    public List<TRecordResp> selectTRecordListWithoutResult(TRecordResultReq tRecord)
    {
        return tRecordMapper.selectTRecordListWithoutResult(tRecord);
    }

    @Override
    public TRecordResp selectById(Long id)
    {
        return tRecordMapper.selectById(id);
    }

    @Override
    public List<TRecordResp> queryRecordListByPHDids(TPatientHospitalDoctor tPatientHospitalDoctor)
    {
        return tRecordMapper.queryRecordListByPHDids(tPatientHospitalDoctor);
    }

    @Override
    public AjaxResult getHistoryList(TRecord tRecord)
    {
        List<TRecord> tRecords = tRecordMapper.selectTRecordList(tRecord);
        tRecords.removeIf(o->"0".equals(o.getStatus()));
        if (CollectionUtils.isEmpty(tRecords)){
            return AjaxResult.success();
        }
        List<TRecord> records = tRecords.stream().sorted(Comparator.comparing(TRecord::getCreateTime)).collect(Collectors.toList());
        List<String> jsonKeyList = new ArrayList<String>(){{
            add("shemian-sz_avg_red");
            add("st_avg_yellow");
            add("sx_vein_color");
        }};
        RecordHistoryDataResp recordHistoryDataRespMap = getDataList(records,jsonKeyList);
        return AjaxResult.success(recordHistoryDataRespMap);
    }

    private RecordHistoryDataResp getDataList(List<TRecord> records, List<String> jsonKeyList)
    {

        RecordHistoryDataResp recordHistoryDataResp = new RecordHistoryDataResp();
        List<Date> dateList = records.stream().map(TRecord::getCreateTime).collect(Collectors.toList());
        recordHistoryDataResp.setDateList(dateList);
        List<JSONObject> result = records.stream().map(tRecord -> {
            JSONObject jsonObject;
            if(StringUtils.isEmpty(tRecord.getUpdateResult2())){
                jsonObject = JSONObject.parseObject(tRecord.getAiResult2());
            }else {
                jsonObject = JSONObject.parseObject(tRecord.getUpdateResult2());
            }
            return jsonObject;
        }).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(result)){
            List<Integer> shezhiList = new LinkedList<>();
            List<Integer> shetaiList = new LinkedList<>();
            List<Integer> shexiaList = new LinkedList<>();
            String[] keys0 = jsonKeyList.get(0).split("-");
            String key1 = jsonKeyList.get(1);
            String key2 = jsonKeyList.get(2);
            result.forEach(jsonObject -> {
                shezhiList.add(jsonObject.getJSONObject("shezhi").getJSONObject(keys0[0]).getInteger(keys0[1]));
                shetaiList.add(jsonObject.getJSONObject("shetai").getInteger(key1));
                shexiaList.add(jsonObject.getJSONObject("shexia").getInteger(key2));
            });
            recordHistoryDataResp.setShezhiList(shezhiList);
            recordHistoryDataResp.setShetaiList(shetaiList);
            recordHistoryDataResp.setShexiaList(shexiaList);
        }
        return recordHistoryDataResp;
    }
}
